#ifndef AlienCommand_h
#define AlienCommand_h

// $Id: AlienCommand.h,v 1.20 2014/09/12 08:43:12 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "Debug.h"
#include "QOrderedSet.h"

#include <QString>
#include <QStringList>
#include <QHash>
#include <QList>

//! handles top commands
class AlienCommand: public Counter
{

    public:

    //! enumeration for command type
    enum Type
    {

        None,
        Connect,
        CheckProxy,
        FlushErrors,
        DefaultUser,
        DefaultMasterList,
        MasterList,
        ChildrenList,
        Kill,
        KillMaster,
        Resubmit,
        ResubmitMaster,
        Trace,
        TraceJdl,
        TraceAll,
        JobFiles,
        Copy,
        Remove,
        RemoveDirectory,
        UserList,
        ClearQueue
    };

    //! list of commands
    using List = QList<AlienCommand>;

    //! constructor
    AlienCommand( const Type = None );

    //! less than operator
    virtual bool operator < ( const AlienCommand& other ) const
    {
        if( priority() != other.priority() ) return priority() < other.priority();
        else return id() < other.id();
    }

    //! equal to operator
    bool operator == (const AlienCommand& other ) const
    { return priority() == other.priority() && id() == other.id(); }

    //!@name accessors
    //@{

    //! command name
    QString name( void ) const
    { return name( type() ); }

    //! map Type to command name
    static QString name( const Type& type );

    //! id
    int id( void ) const
    { return id_; }

    //! priority
    int priority( void ) const
    { return priority_; }

    //! type
    Type type( void ) const
    { return type_; }

    // true if arguments are passed
    bool hasArguments( void ) const
    { return !arguments_.isEmpty(); }

    //! arguments
    const QStringList& arguments( void ) const
    { return arguments_; }

    //! user
    QString user( void ) const;

    //! master job id
    int parentJobId( void ) const;

    //! job id
    int jobId( void ) const;

    //! path
    QString path( void ) const;

    //! return error message associated to this command, if failed
    QString errorString( void ) const;

    //! error details
    QString errorDetails( void ) const
    { return details_; }

    //@}

    //!@name modifiers
    //@{

    //! priority
    void setPriority( unsigned int value )
    { priority_ = value; }

    //! add argument
    AlienCommand& addArgument( const QString& argument )
    {
        arguments_ << argument;
        return *this;
    }

    //! push arguments
    AlienCommand& operator << (const QString& argument )
    { return addArgument( argument ); }

    //! clear arguments
    void clearArguments( void )
    { arguments_.clear(); }

    //! set error details
    void setErrorDetails( const QString& value )
    { details_ = value; }

    //@}

    private:

    //! static id counter
    static unsigned int& _counter( void );

    //! map types to command name
    using NameMap = QHash<Type,QString>;

    //! name map
    static NameMap& _names( void );

    //! unique id
    int id_;

    //! priority
    int priority_;

    //! type
    Type type_;

    //! details (in case of errors)
    QString details_;

    //! arguments
    QStringList arguments_;

    //! initialize command names
    static void _initializeNames( void );

};

//! used to sort commands based on priority and type
class SortedAlienCommand: public AlienCommand
{

    public:

    //! constructor
    SortedAlienCommand( void )
    {}

    //! constructor
    SortedAlienCommand( const AlienCommand& other ):
        AlienCommand( other )
        {}

    virtual bool operator < (const SortedAlienCommand& other ) const
    {

        if( priority() != other.priority() ) return priority() < other.priority();
        if( type() != other.type() ) return type() < other.type();

        // check arguments
        for( int index = 0; index < std::min( arguments().size(), other.arguments().size() ); ++index )
        { if( arguments()[index] != other.arguments()[index] ) return arguments()[index] < other.arguments()[index]; }

            // check arguments size
        return arguments().size() < other.arguments().size();

    };

    using Set = QOrderedSet<SortedAlienCommand>;

};

#endif
