#ifndef AlienInterface_h
#define AlienInterface_h

// $Id: AlienInterface.h,v 1.39 2014/09/12 08:43:12 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Debug.h"
#include "AlienCommand.h"
#include "AlienCommandList.h"
#include "File.h"
#include "FileRecord.h"
#include "Job.h"
#include "JobSet.h"
#include "MessageBuffer.h"
#include "TextFormat.h"
#include "UserSet.h"

#include <QObject>
#include <QProcess>
#include <QProcessEnvironment>
#include <QString>
#include <QHash>

class CustomProcess;

//! Job managment
class AlienInterface:public QObject, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    AlienInterface( QObject* parent = 0 );

    //! destructor
    virtual ~AlienInterface( void );

    //! find certificates
    void findCertificates( void );

    //! clear failed commands
    void clearFailedCommands( void )
    { failedCommands_.clear(); }

    //! hide killed jobs
    void setHideKilledJobs( bool value )
    { hideKilledJobs_ = value; }

    //! true if connected
    bool isConnected( void ) const
    { return connected_; }

    Q_SIGNALS:

    //! connection lost
    void connectionLost( const QString& = QString() );

    //! connection state changed
    void connectionStateChanged( bool );

    //! default user is available
    void defaultUserAvailable( QString );

    //! user list is available
    void userListAvailable( UserSet );

    //! master list is available
    void masterListAvailable( QString, const JobSet& );

    //! children list is available
    void childrenListAvailable( const Job&, const JobSet& );

    //! job information is available
    void jobInformationAvailable( const Job&, const QString& = QString() );

    //! children information is available
    void childrenInformationAvailable( const Job&, const JobSet& );

    //! job output files
    void jobFilesAvailable( const Job&, const QString&, const FileRecord::List& );

    //! job JDL is available
    void jdlAvailable( const Job&, const QString& );

    //! command added
    void commandAdded( const AlienCommand& );

    //! commands added
    void commandsAdded( const AlienCommand::List& );

    //! command started
    void commandStarted( const AlienCommand& );

    //! command completed
    void commandCompleted( const AlienCommand& );

    //! commands failed
    void commandsFailed( const AlienCommand::List& );

    //! emitted when busy state is changed
    void busyStateChanged( bool );

    //! emmitted when busy processing commands
    void busy( void );

    //! emmitted when there is no more commands in pile
    void ready( void );

    //! logAvailable
    void logAvailable( const QString&, Format::TextFormatFlags = 0 );

    //! message
    void messageAvailable( const QString& );

    public Q_SLOTS:

    //! add command
    void addCommand( const AlienCommand& );

    //! add commands
    void addCommands( const AlienCommand::List& );

    //! abort all commands
    void abortAll( void );

    //! abort selected commands
    void abortCommands( const AlienCommand::List& );

    //! toggle pause
    void togglePause( bool value )
    {
        paused_ = value;
        if( !paused_ ) _runCommand();
    }

    //! update set of active users
    void updateActiveUsers( UserSet users )
    { activeUsers_ = users; }

    //! clear queue
    void clearQueue( const QString&, const Job::IdSet& );

    protected Q_SLOTS:

    //! run next command in pile
    bool _runCommand( void );

    //! take action when current command completed
    void _commandCompleted( int, QProcess::ExitStatus );

    //! error from process
    void _error( QProcess::ProcessError );

    //! read on-fly from process
    void _readStdout( void );

    //! read on-fly from process
    void _readStderr( void );

    protected:

    //! check connection
    bool _checkConnection( void );

    //! process proxy
    void _processProxyInfo( void );

    //! process default user
    void _processDefaultUser( void );

    //! process user list
    void _processUserList( void );

    //! process master job list
    void _processMasterList( const QStringList& );

    //! process children job list
    void _processChildrenList( const Job& );

    //! job output files
    void _processJobFiles( const Job&, const QString& );

    //! process job set
    void _processJobInformation( Job, bool );

    //! process job set
    void _processJdl( const Job& );

    //! reset buffers
    void _resetBuffers( void );

    //! reset process
    void _resetProcess( void );

    //! create new process to run command
    CustomProcess* _newProcess( void );

    //! attempt at identifying error lines
    bool _isError( const QString& ) const;

    //! find killed job in stored map
    /*! returns invalid job if not found */
    Job _findKilledJob( const QString&, int ) const;

    //! true if given error is to be ignored
    bool _ignoreError( const AlienCommand&, const QString& ) const;

    private Q_SLOTS:

    //! update configuration
    void _updateConfiguration( void );


    private:

    //! true if paused
    bool paused_;

    //! hide killed jobs
    bool hideKilledJobs_;

    //! custom process
    CustomProcess* process_;

    //! stdout buffer
    MessageBuffer stdoutBuffer_;

    //! stderr buffer
    MessageBuffer stderrBuffer_;

    //! map command type to name
    AlienCommandList commands_;

    //! failed commands
    AlienCommand::List failedCommands_;

    //! set of active users
    UserSet activeUsers_;

    //! map users to (killed) job set
    using JobMap = QHash< QString, JobSet >;
    JobMap killedJobs_;

    //! system environment
    QProcessEnvironment systemEnvironment_;

    //! true if connection was valid at least once
    bool wasConnected_;

    //! connected flag
    bool connected_;

    //! ignore next error (due to abort)
    bool ignoreErrors_;

};

#endif
