// $Id: Job.cpp,v 1.55 2015/01/05 14:37:10 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Job.h"
#include "JobSet.h"

#include "File.h"
#include "XmlDef.h"

#include <QRegExp>

//____________________________________________________________
Job::FieldVector &Job::_fieldNames( void )
{
    static FieldVector singleton( nFields, QString() );
    return singleton;
}

//____________________________________________________________
Job::FieldMap& Job::_fieldMap( void )
{
    static FieldMap singleton;
    return singleton;
}

//____________________________________________________________
void Job::installFields( void )
{
    Debug::Throw( "Job::installFields.\n" );
    _setFieldName( Id, QObject::tr( "Id" ) );
    _setFieldName( User, QObject::tr( "Owner " ) );
    _setFieldName( Status, QObject::tr( "Status" ) );
    _setFieldName( Command, QObject::tr( "Command " ) );
    _setFieldName( Resubmitted, QObject::tr( "Resubmitted " ) );
    _setFieldName( SubmissionTime, QObject::tr( "Submitted" ) );
    _setFieldName( StartTime, QObject::tr( "Started" ) );
    _setFieldName( EndTime, QObject::tr( "Completed" ) );
    return;
}

//____________________________________________________________
Job::ConversionMap &Job::_conversions( void )
{
    static ConversionMap singleton;
    return singleton;
}

//____________________________________________________________
void Job::installConversions( void )
{
    Debug::Throw( "Job::installConversions.\n" );
    _conversions().insert( "WAITING", "W" );
    _conversions().insert( "INSERTING", "I" );
    _conversions().insert( "RUNNING", "R" );
    _conversions().insert( "STARTED", "ST" );
    _conversions().insert( "SAVING", "SV" );
    _conversions().insert( "SAVED", "SVD" );
    _conversions().insert( "DONE", "D" );
    _conversions().insert( "ERROR_V", "EV" );
    _conversions().insert( "ERROR_VN", "EV" );
    _conversions().insert( "ERROR_SV", "ESV" );
    _conversions().insert( "ERROR_E", "EE" );
    _conversions().insert( "ERROR_RE", "ERE" );
    _conversions().insert( "ZOMBIE", "Z" );
}

//____________________________________________________________
Job::Job( const QDomElement& element ):
    Counter( "Job" ),
    parentId_( -1 ),
    fields_( nFields )
{

    Debug::Throw( "Job::Job (dom).\n" );

    // parse attributes
    QDomNamedNodeMap attributes( element.attributes() );
    for( int i=0; i<attributes.count(); i++ )
    {
        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;
        if( attribute.name() == Xml::Parent ) setParentId( attribute.value().toInt() );
        else Debug::Throw(0) << "Job::Job (dom) - unrecognized attribute: " << attribute.name() << endl;
    }

    // parse children elements
    for(QDomNode childNode = element.firstChild(); !childNode.isNull(); childNode = childNode.nextSibling() )
    {
        QDomElement childElement = childNode.toElement();
        QString tagName( childElement.tagName() );
        if( tagName == Xml::Field ) _loadXmlField( childElement );
        else Debug::Throw(0) << "Job::Job - unrecognized child tagName " << tagName << endl;
    }

}

//____________________________________________________________
Job::Job( const QString& buffer ):
    Counter( "Job" ),
    parentId_( -1 ),
    fields_( nFields )
{
    // split buffer
    QStringList fields( buffer.split( QRegExp("\\s+") ) );
    if( fields.size() == 3 )
    {

        assign( Id, fields[1] );
        assign( User, fields[0] );
        assign( Status, fields[2] );

    } else if( fields.size() == 4 ) {

        assign( Id, fields[0] );

        // need to convert status to matching short string
        // todo: cool use conversion map for speeding up
        const QString status( fields[1] );
        ConversionMap::const_iterator iter( _conversions().find( status ) );
        assign( Status, iter == _conversions().end() ? status:iter.value() );

        assign( Command, File( fields[2] ).localName() );

    }


    return;
}

//____________________________________________________________
QDomElement Job::domElement( QDomDocument& document ) const
{

    QDomElement out = document.createElement( Xml::Job );
    out.setAttribute( Xml::Parent, QString::number( parentId() ) );

    // add fields
    for( int index=0; index<fields_.size(); ++index )
    {

        if( !has( index ) ) continue;
        QDomElement field = out.appendChild( document.createElement( Xml::Field ) ).toElement();
        field.setAttribute( Xml::FieldName, _fieldNames()[index] );
        field.appendChild( document.createTextNode( fields_[index] ) );

    }

    return out;

}

//____________________________________________________________
QString Job::status( void ) const
{

    if( isFailed() ) return QString( QObject::tr( "Failed (%1)" ) ).arg( error() );
    else {

        QString st( get( Status ) );
        if( id() == 0 ) return "";
        else if( st.isEmpty() ) return QObject::tr( "Unknown" );
        else if( st == "I" ) return QObject::tr( "Inserting" );
        else if( st == "SP" || st == "WS" ) return QObject::tr( "Splitting" );
        else if( st == "R" || st == "RS" ) return QObject::tr( "Running" );
        else if( st == "D" ) return QObject::tr( "Completed" );
        else if( st == "DONE_WARN" ) return QObject::tr( "Completed (warning)" );
        else if( st == "SAVED_WARN" ) return QObject::tr( "Saved (warning)" );
        else if( st == "W" ) return QObject::tr( "Waiting" );
        else if( st == "OVER_WAITING" ) return QObject::tr( "Over waiting" );
        else if( st == "ASSIGNED" ) return QObject::tr( "Assigned" );
        else if( st == "S" ) return QObject::tr( "Split" );
        else if( st == "SV" ) return QObject::tr( "Saving" );
        else if( st == "SVD" ) return QObject::tr( "Saved" );
        else if( st == "ST" ) return QObject::tr( "Starting" );
        else if( st == "K" ) return QObject::tr( "Killed" );
        else if( st == "Z" ) return QObject::tr( "Zombie" );
        else return st;
    }
}

//____________________________________________________________
QString Job::error( void ) const
{

    QString st( get( Status ) );
    if( st.isEmpty() ) return QString();
    else if( st == "EI" ) return QObject::tr( "insertion" );
    else if( st == "EA" ) return QObject::tr( "assignment" );
    else if( st == "ES" ) return QObject::tr( "queuing" );
    else if( st == "EE" ) return QObject::tr( "execution" );
    else if( st == "ER" ) return QObject::tr( "running" );
    else if( st == "EV" ) return QObject::tr( "validation" );
    else if( st == "EIB" ) return QObject::tr( "input box" );
    else if( st == "ESV" ) return QObject::tr( "saving" );
    else if( st == "ESP" ) return QObject::tr( "splitting" );
    else if( st == "ERE" ) return QObject::tr( "reading" );
    else if( st == "EXPIRED" ) return QObject::tr( "expired" );
    else return st;

}

//____________________________________________________________
QString Job::get( int index ) const
{
    if( index < nFields ) return fields_[index];
    else return QString::number( childrenStatistics_.get( index - nFields ) );
}

//____________________________________________________________
Job& Job::assign( int index, const QString& value )
{
    fields_[index] = value;
    switch( index )
    {
        case SubmissionTime: submissionTime_ = _time( value ); break;
        case StartTime: startTime_ = _time( value ); break;
        case EndTime: endTime_ = _time( value ); break;
        default: break;
    }
    return *this;
}

//____________________________________________________________
void Job::incrementResubmission( void )
{
    const int resubmission( has( Resubmitted ) ? get<int>( Resubmitted ):0 );
    assign<int>( Resubmitted, resubmission+1 );
}

//____________________________________________________________
bool Job::isInserting( void ) const
{ return get( Status ) == "I"; }

//____________________________________________________________
bool Job::isRunning( void ) const
{
    const QString status( get( Status ) );
    return
        (isMasterJob() && status == "I") ||
        status == "SP" || // splitting
        status == "R" ||
        status == "RS" ||
        status == "S" ||
        status == "ST" ||
        status == "SV" ||
        status == "SVD" ||
        status == "SAVED_WARN";
}

//____________________________________________________________
bool Job::isCompleted( void ) const
{
    const QString status( get( Status ) );
    return
        status == "D" ||
        status == "DONE_WARN";
}

//____________________________________________________________
bool Job::isPending( void ) const
{
    const QString status( get( Status ) );
    return
        (hasParent() && status == "I" ) ||
        status == "W" ||
        status == "OVER_WAITING" ||
        status == "ASSIGNED";
}

//____________________________________________________________
bool Job::isFailed( void ) const
{
    const QString status( get( Status ) );
    return status.startsWith( 'E' ) || status == "Z";
}

//____________________________________________________________
bool Job::isKilled( void ) const
{ return get( Status ) == "K"; }

//____________________________________________________________
bool Job::mightHaveChildren( void ) const
{

    // list of status values corresponding to master jobs that have no children
    static QSet<QString> statusSet;
    if( statusSet.empty() )
    {
        statusSet
            << "W" << "R" << "I"
            << "SV" << "SVD"
            << "K"
            << "EE" << "ER" << "EV"
            << "ESV" << "EIB"
            << "Z" << "EXPIRED";
    }

    // true if status is not in above list
    return !statusSet.contains( get( Status ) );

}

//____________________________________________________________
QString Job::resubmitted( void ) const
{
    if( !has( Job::Resubmitted ) ) return QString();
    else {

        const int count( get<int>( Job::Resubmitted ) );
        return count == 1 ? QString( QObject::tr( "%1 time" ) ).arg( count ) : QString( QObject::tr( "%1 times" ) ).arg( count );

    }

}

//____________________________________________________________
void Job::_loadXmlField( const QDomElement& element )
{
    Debug::Throw( "Job::_loadXmlField.\n" );
    QDomNamedNodeMap attributes( element.attributes() );
    int id( 0 );
    for( int i=0; i<attributes.count(); i++ )
    {
        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;
        QString name( attribute.name() );
        QString value( attribute.value() );

        if( name != Xml::FieldName )
        {

            Debug::Throw(2) << "Job::_loadXmlField - unrecognized attribute: " << name << endl;
            continue;

        } else if( !_xmlToId( value, id ) ) {

            Debug::Throw(2) << "Job::_loadXmlField - field: " << value << " rejected" << endl;
            return;

        } else {

            Debug::Throw(2) << "Job::_loadXmlField - field: " << value << " accepted" << endl;
            break;

        }
    }

    // get node text and assign
    assign( id, element.text() );

}

//____________________________________________________________
TimeStamp Job::_time( const QString& value )
{

    QRegExp regExp( "(\\w+)\\s+(\\d+)\\s+(\\d+):(\\d+)" );
    const int index( regExp.indexIn( value ) );
    if( index < 0 || regExp.captureCount() < 4 ) return TimeStamp();

    const QStringList captures( regExp.capturedTexts() );
    TimeStamp out( TimeStamp::now() );
    out.setMonth( captures[1] )
        .setDay( captures[2].toInt() )
        .setHours( captures[3].toInt() )
        .setMinutes( captures[4].toInt() ).makeTime();

    return out;
}

//____________________________________________________________
Job::IdSet::IdSet( const JobSet& jobs ):
    Counter( "Job::IdSet" )
{
    foreach( const Job& job, jobs )
    { if( job.id() >= 0 ) insert( job.id() ); }
}

//____________________________________________________________
Job::IdSet::IdSet( const Job::List& jobs ):
    Counter( "Job::IdSet" )
{
    foreach( const Job& job, jobs )
    { if( job.id() >= 0 ) insert( job.id() ); }
}
